<h4 class="defcon{{overall_severity}}" title="{{detector_descr}}">
 <p class="left">detector: {{ detector_name }}</p> 
 <span class="defcon{{overall_severity}} dc" title="overall rating; 1=worst 5=best">DC:{{overall_severity}}</span>
</h4>
<div class="detector score">
 <p class="left"><span>absolute score:</span> <b class="defcon{{absolute_defcon}}">{{ absolute_score }} ({{absolute_comment}})</b></p>
 <span class="defcon{{absolute_defcon}} dc" title="rating; 1=worst 5=best">DC:{{absolute_defcon}}</span>
</div>
{%if zscore != "n/a"%}
<div class="detector score">
 <p class="left"><span>relative score (Z):</span> <b class="defcon{{zscore_defcon}}">{{zscore}} ({{zscore_comment}})</b></p>
 <span class="defcon{{zscore_defcon}} dc" title="rating; 1=worst 5=best">DC:{{zscore_defcon}}</span>
</div>
{%else%}
<div class="detector score">
 <p class="left"><span>relative score (Z):</span> unavailable, calibration not present for this probe:detector combination</p> 
 <span class="dc" title="DEFCON rating; 1=worst 5=best">n/a</span>
</div>
{%endif%}