
<button class="defcon{{severity}} accordion"><b>{{ group }}</b>{% if show_top_group_score %} <span title="{{group_aggregation_function}}">- {{ group_score }} </span>{%endif%}</button>
<div class="panel">
<!-- <p>{{doc}}</p> -->
{%if group_score != "100.0%"%}
<p>
{%if group_link%}
<a href="{{group_link}}" target="_new">{{group}}</a>
{%else%}
"{{group}}"
{%endif%}
</p>
{%endif%}