# SPDX-FileCopyrightText: Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
This module implements provide functionality useful in red team engagement involving LLMs.
Functionality enables building blocks for reuse in techniques that utilized LLMs directly and
indirectly in targeted LLM testing.

Some of the code in this module is derived from Robust Intelligence's TAP implementation: https://github.com/RICommunity/TAP
"""
